use strict;
use warnings;
use List::Util qw/shuffle/;

my ($in_fasta_scaffolds, $in_tara_contigs_index, $out_fasta_chopped_scaffolds) = @ARGV;

my $contig_lengths_ref = get_contig_lengths($in_tara_contigs_index);
my $scaffolds_ref = read_scaffolds($in_fasta_scaffolds);
wwrite_fasta_chopped_scaffolds($scaffolds_ref, $contig_lengths_ref, $out_fasta_chopped_scaffolds);

sub get_contig_lengths {
	my ($in_tara_contigs_index) = @_;
	my @contig_lengths;
	open (my $in, "<", $in_tara_contigs_index) or die "could not open $in_tara_contigs_index for reading\n";
	my $line = <$in>;
	while (defined $line) {
		chomp ($line);
		my @line_parts = split (/\s+/, $line);
		push (@contig_lengths, $line_parts[2]);
		$line = <$in>;
	}
	close ($in);
	return (\@contig_lengths);
}

sub read_scaffolds {
	my ($in_fasta_scaffolds) = @_;
	my %scaffolds;
	open (my $in, "<", $in_fasta_scaffolds) or die "could not open $in_fasta_scaffolds for reading\n";
	my $line = <$in>;
	while (defined $line) {
		chomp ($line);
		if ($line =~ m/^>(.*)/) {
			my $header = $1;
			my $seq = "";
			$line = <$in>;
			while ((defined $line) and ($line !~ m/^>/)) {
				chomp ($line);
				$seq .= $line;
				$line = <$in>;
			}
			$scaffolds{$header} = $seq;
		}
	}
	close ($in);
	return (\%scaffolds);
}

sub wwrite_fasta_chopped_scaffolds {
	my ($scaffolds_ref, $contig_lengths_ref, $out_fasta_chopped_scaffolds) = @_;
	open (my $out, ">", $out_fasta_chopped_scaffolds) or die "could not open $out_fasta_chopped_scaffolds for writing\n";
	foreach my $scaffold (keys %{$scaffolds_ref}) {
		print "$scaffold was chopped to ";
		my $seq = $scaffolds_ref->{$scaffold};
		my $remaining_length = length($seq);
		my $offset = 0;
		my $num_chops = 0;
		my @shuff_contig_lengths = shuffle(@{$contig_lengths_ref});
		while ($remaining_length > 0) {
			my $length_to_take = pop(@shuff_contig_lengths);
			if ($remaining_length <= $length_to_take) {
				$length_to_take = $remaining_length;
			}
			
			# get a chop of the scaffold:
			my $chopped_seq = substr($seq, $offset, $length_to_take);
			
			# prepare chop's header:
			my $scaffold_low = $offset;
			my $scaffold_high = $offset + $length_to_take - 1;
			my $chopped_header = "$scaffold\;$scaffold_low\;$scaffold_high";
	 		
	 		# write fasta:
			print $out ">$chopped_header\n$chopped_seq\n";
			$num_chops++;
	 
			# prepare for next iteration
			$offset += $length_to_take;
			$remaining_length -= $length_to_take;
		}
		print "$num_chops chops with an average length of: ";
		my $avg = length($seq) / $num_chops;
		print "$avg\n";
	}
	close ($out);
}