library(cowplot)
library(viridis)
library(ggpubr)
library(RColorBrewer)
library("pROC")
library("PRROC")
library(plyr)
library(dplyr)
library(scales)
library(colorspace)
library(ggridges)

theme_set(theme_cowplot(13))

args <- commandArgs(TRUE)
metaeuk_results_path <- as.character(args[1])
evaluation_dir <- as.character(args[2])
path_for_plots <- as.character(args[3])
print(metaeuk_results_path)

## for debug:
metaeuk_results_path="annotatedSpecs/mmseqs_ebb1_metaeuk_4714/"
evaluation_dir="evaluation"
path_for_plots="annotatedSpecs/mmseqs_ebb1_metaeuk_4714/"

organism_names = c("Spombe", "Acaste", "Phyto", "Bbige", "Locea", "Ptric", "Anidu")
tot_num_proteins = c(5132, 14974, 6381, 5079, 668, 10408, 9556)
base_spec_colors = list(
  "Spombe" = "#F8766D",
  "Acaste" = "#C49A00",
  "Phyto"  = "#53B400",
  "Bbige"  = "#00C094",
  "Locea"  = "#00B6EB",
  "Ptric"  = "#A58AFF",
  "Anidu"  = "#FB61D7",
  "grey"   = "#666666"
)


## define lighter shades for each species to color exons:
lighter_spec_colors = list()
lighter_count = length(organism_names)
for (i in names(base_spec_colors)) {
  base = base_spec_colors[[i]]
  colors = c(base)
  last = base
  for (j in 1:(lighter_count - 1)) {
    lighter_color = lighten(last, 0.25)
    last = lighter_color
    colors = c(colors, lighter_color)
  }
  lighter_spec_colors[[i]] = rev(colors) 
}

## define a gray scale of shades:
fake_grey = data.frame(x = seq(1, length(lighter_spec_colors[["grey"]])), y = factor( 
  x = c(0, 1, 2, 3, 4, 5, 6),
  labels = rev(c("100%", "90%", "80%", "70%", "60%", "50%", "<50%")),
  levels = c(0, 1, 2, 3, 4, 5, 6)
))
exon_covered_legend = get_legend(ggplot(fake_grey, aes(x, fill = y)) +
                                   geom_bar() +
                                   scale_fill_manual(name="Covered annot. exons", values=lighter_spec_colors[["grey"]]) +
                                   guides(fill=guide_legend(ncol=1)))

# convert short string to scientific name:
get_scientific_name = function(organism_name)
{
  label = ""
  if (organism_name == "Spombe") {
    label = "S. pombe"
  }
  if (organism_name == "Acaste") {
    label = "A. castellanii"
  }
  if (organism_name == "Bbige") {
    label = "B. bigemina"
  }
  if (organism_name == "Phyto") {
    label = "Phytomonas EM1"
  } 
  if (organism_name == "Locea") {
    label = "L. oceanica"
  }
  if (organism_name == "Ptric") {
    label = "P. tricornutum"
  }
  if (organism_name == "Anidu") {
    label = "A. nidulans"
  }
  return (label)
}


### three lines for debugging: ### 
organism_name=organism_names[4]
total_num_proteins_organism=tot_num_proteins[4]
organism_index=4
### ### ### ### ### ### ### ### ### 


##################

draw_sensitivity_at_distance = function(organism_names,tot_num_proteins) {
  seqIDs = c(40,60,80,90,100)
  
  legend_labels = organism_names
  
  sensitvity_df = data.frame("organism" = factor(rep(organism_names, each = length(seqIDs)), levels = organism_names), 
                             "min_seqid_member_to_T" = rep(seqIDs, length(organism_names)), 
                             "prop_discovered_annot_prots" = rep(0,length(organism_names)*length(seqIDs)))
  
  for (j in 1:length(organism_names)) {
    organism_name = organism_names[j]
    total_num_proteins_organism = tot_num_proteins[j]
    legend_labels[j] = paste0(get_scientific_name(organism_name), " (", format(x = total_num_proteins_organism, scientific = FALSE, big.mark = ",", big.interval = 3), ")")
    
    basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
    metaeuk_predRed_res_file = paste(basic_path, "predRed_matched_to_annot_prot.tsv",sep = "")
    metaeuk_predRed_res_df = read.table(file = metaeuk_predRed_res_file, header = TRUE, sep = "\t")
    metaeuk_predRed_res_df$annotProt = as.character(metaeuk_predRed_res_df$annotProt)
    
    for (i in 1:length(seqIDs)) {
      x = seqIDs[i] / 100
      metaeuk_predRed_res_reduced_df = metaeuk_predRed_res_df[which(metaeuk_predRed_res_df$min_seqid_group <= x),]
      sensitvity_df[(length(seqIDs)*(j - 1) + i),3] = length(unique(metaeuk_predRed_res_reduced_df$annotProt)) / total_num_proteins_organism
    }
  }

  # prepare figure
  fig = ggplot(sensitvity_df, aes(x=min_seqid_member_to_T, y=prop_discovered_annot_prots, color=organism, shape=organism)) + 
    geom_line(linetype = "dashed", size=1) +
    geom_point(size=4) +
    xlab(expression("Max. seq. id to target")) +
    ylab(expression("Sensitvity")) +
    scale_x_reverse(breaks = seqIDs, labels = paste0(seqIDs,"%")) +
    scale_y_continuous(limits = c(0,1), breaks = seq(0, 1, by=0.25), labels = scales::percent) +
    scale_color_discrete(name = "Organism\n(annotated proteins)", labels = legend_labels) +
    scale_shape_manual(name = "Organism\n(annotated proteins)", labels = legend_labels, values = c(15,16,18,17,18,19,20)) +
    theme(legend.position = c(0.2, 0.2), legend.text = element_text(face = "italic"), legend.title = element_text(hjust = 0.5))
  
  return (fig)
}
##################

draw_covered_exons = function(organism_name,total_num_proteins_organism) {
  basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
  metaeuk_predRed_res_file = paste(basic_path, "predRed_matched_to_annot_prot.tsv",sep = "")
  metaeuk_predRed_res_df = read.table(file = metaeuk_predRed_res_file, header = TRUE, sep = "\t")
  
  hist_df = metaeuk_predRed_res_df[,c(8,10,12)]
  hist_df$prop_discovered = (hist_df$num_shared_CDSs + hist_df$num_covered_CDSs_annot) / hist_df$num_CDSs_annot
  hist_df$prop_cat = round(hist_df$prop_discovered, digits = 1)
  # change values to fit drawing
  hist_df$prop_cat[which(hist_df$prop_cat < 0.5)] = 0
  hist_df$prop_cat = factor(100*hist_df$prop_cat, levels = (c(0,seq(50,100,by=10))), labels = (c("<50%",paste0(seq(50,100,by=10),"%"))))

  hist_df$cat_num_CDSs_annot = hist_df$num_CDSs_annot
  hist_df$cat_num_CDSs_annot[which(hist_df$cat_num_CDSs_annot > 5)] = 6
  hist_df$cat_num_CDSs_annot = factor(hist_df$cat_num_CDSs_annot, levels = seq(1:6), labels = c(seq(1:5),"6+"))
  
  num_genes_labels = scales::comma(plyr::count(hist_df$cat_num_CDSs_annot)$freq)
  
  fig = ggplot(hist_df, aes(cat_num_CDSs_annot, fill = prop_cat)) +
    geom_bar(position = "fill") +
    scale_x_discrete(expand=c(0,0), drop=FALSE) +
    scale_y_continuous(expand=c(0,0), labels = scales::percent) +
    scale_fill_manual(name="Covered annot. exons", values = lighter_spec_colors[[organism_name]], drop=FALSE) + 
    xlab(expression("")) +
    ylab(expression("")) +
    annotate("text", x = seq(1:length(num_genes_labels)), y = 1.05, label = num_genes_labels, vjust = 1) +
    guides(fill=guide_legend(ncol=2)) +
    theme(legend.position = "none")
  
  return (fig)
}

##################

draw_exon_num_length_dist = function(organism_name) {
  basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
  metaeuk_exon_len_file = paste(basic_path, "predRed_exon_num_len.tsv",sep = "")
  metaeuk_exon_len_df = read.table(file = metaeuk_exon_len_file, header = TRUE, sep = "\t")
  
  lengthExonsSingleExonPreds = metaeuk_exon_len_df$length_each_exon[which(metaeuk_exon_len_df$num_exons == 1)]
  lengthExonsMultiExonPreds = metaeuk_exon_len_df$length_each_exon[which(metaeuk_exon_len_df$num_exons > 1)]
  lengthExonsDf = data.frame(LENGTH = c(lengthExonsSingleExonPreds,lengthExonsMultiExonPreds), GROUP = c(rep("Single",length(lengthExonsSingleExonPreds)),rep("Multi",length(lengthExonsMultiExonPreds))))

  figExonsLength = ggplot(lengthExonsDf, aes(x=log(LENGTH), y=GROUP, fill=GROUP, color=GROUP)) + 
    geom_density_ridges(quantile_lines = TRUE, alpha = 0.5, scale = 100000) +
    scale_x_continuous(limits = log(c(9,3000)), breaks = log(c(10,100,1000)), labels = comma(c(10,100,1000))) +
    xlab("") +
    ylab("") +
    scale_y_discrete(breaks = NULL) +
    scale_fill_discrete (
      "",
      breaks=c("Single", "Multi"),
      labels=c("1 putative exon", "2+ putative exons")
    ) +
    scale_color_discrete (
      breaks=NULL
    )
  
  numPredsSingleExon = length(which(metaeuk_exon_len_df$num_exons == 1))
  numPredsTwoExons = length(which(metaeuk_exon_len_df$num_exons == 2)) / 2
  totalNumPreds = length(unique(as.numeric(metaeuk_exon_len_df$pred_id)))
  numPredsThreeOrMoreExons = totalNumPreds - (numPredsTwoExons + numPredsSingleExon)
  
  predsByNumExonsDf = data.frame("NUM_EXONS" = c(1,2,3), "PRED_COUNT" = c(numPredsSingleExon, numPredsTwoExons, numPredsThreeOrMoreExons))
  totNumPreds = sum(predsByNumExonsDf$PRED_COUNT)
  predsByNumExonsDf$PERCENT_PREDS = round(100 * predsByNumExonsDf$PRED_COUNT / totNumPreds, digits = 2)
  predsByNumExonsDf$COUNT_LABEL = paste0(round((predsByNumExonsDf$PRED_COUNT / 1000), digits = 1),"K")
  
  figPredsByNumExons = ggplot(predsByNumExonsDf, aes(y = PERCENT_PREDS, x = as.factor(NUM_EXONS))) + 
    geom_bar(stat="identity", position = "dodge", width=0.8) + 
    #geom_text(aes(label = COUNT_LABEL, x = as.factor(NUM_EXONS)), position = position_dodge(width = 0.8), vjust = -0.6, size = 3.5) +
    annotate("text", x = 2, y = 90, label = get_scientific_name(organism_name), fontface = 'italic') +
    scale_y_continuous(breaks = seq(0, 100, by = 20), limits = c(0,100), labels = rep("",6), expand = c(0,0)) +
    scale_x_discrete(breaks = c(1,2,3), labels = c("1","2","3+")) +
    xlab("") + 
    ylab("")
  
  return(list(NUM_EXONS = figPredsByNumExons, EXON_LEN = figExonsLength))
}

#################

draw_redundancy_reduction = function(organism_names)
{
  labels_vector = c()
  organism_vector = c()
  percent_vector = c()
  
  for (i in 1:length(organism_names)) {
    organism_name = organism_names[i]
    
    basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
    metaeuk_predRed_res_file = paste(basic_path, "predRed_matched_to_annot_prot.tsv",sep = "")
    metaeuk_predRed_res_df = read.table(file = metaeuk_predRed_res_file, header = TRUE, sep = "\t")
    
    counts_NCBI_homologs = sort(table(metaeuk_predRed_res_df$annotProt))
    counts_group_sizes = table(counts_NCBI_homologs)
    
    # aggregate:
    counts_group_sizes_df = as.data.frame(counts_group_sizes)
    counts_group_sizes_agg_df = aggregate(counts_group_sizes_df$Freq, by=list(Category=counts_group_sizes_df$counts_NCBI_homologs), FUN=sum)
    
    percent_group_sizes = counts_group_sizes_agg_df$x / sum(counts_group_sizes_agg_df$x)
    labels_of_groups = counts_group_sizes_agg_df$Category
    
    percent_vector = c(percent_vector, percent_group_sizes)
    labels_vector = c(labels_vector, as.numeric(labels_of_groups))
    organism_vector = c(organism_vector, rep(as.character(organism_name),length(percent_group_sizes)))
  }
  
  organism_vector = factor(as.character(organism_vector), levels = as.character(organism_names), labels = sapply(organism_names,get_scientific_name))
  levels(labels_vector) = rev(sort(unique(as.numeric(labels_vector))))
  labels_vector = factor(labels_vector,levels(labels_vector))
  
  df = data.frame(labels_vector, percent_vector, organism_vector)
  
  num_levels = length(levels(labels_vector))
  
  gray_cols = gray.colors(num_levels, start = 0, end = 0.85)
  
  figure = ggplot() + 
    geom_bar(aes(y = percent_vector, x = organism_vector, fill = labels_vector), data = df, stat="identity", width=0.75) + 
    scale_y_continuous(breaks = seq(0, 1, by = 0.2), limits = c(0,1.05), labels = scales::percent, expand=c(0,0)) +
    scale_fill_manual(name="# MetaEuk preds.", values = gray_cols, guide = guide_legend(reverse=TRUE)) +
    xlab("") + 
    ylab("Discovered annotated proteins") + 
    theme(
      axis.text.y = element_text(face = 'italic'),
      legend.position = "top",
      legend.box.margin=margin(c(5,0,-5,-75)),
      legend.direction='horizontal'
    ) +
    coord_flip()
  
  return (figure)
}

##################

get_ggroc_df = function(rocs) {
  RocVals = plyr::ldply(names(rocs), function(rocName) {
    data.frame(
      fpr = 1 -rocs[[rocName]]$specificities,
      tpr = rocs[[rocName]]$sensitivities,
      name = rocName
    )
  })
  return(RocVals %>% group_by(name, fpr) %>% summarise(min_tpr=min(tpr),max_tpr=max(tpr)))
}

##################

draw_mapping_roc = function(organism_names) {
  
  legend_labels = list()
  roc_list = list()
  
  for (j in 1:length(organism_names)) {
    organism_name = organism_names[j]
    
    basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
    metaeuk_predRed_res_file = paste(basic_path, "predRed_map_to_annots.tsv",sep = "")
    metaeuk_predRed_res_df = read.table(file = metaeuk_predRed_res_file, header = TRUE, sep = "\t")
    
    metaeuk_predRed_res_df$Type_cat = 0
    metaeuk_predRed_res_df$Type_cat[which(metaeuk_predRed_res_df$Type == "mapped_to_prot")] = 1
    
    roc_df = data.frame("metaeuk_evalue" = metaeuk_predRed_res_df$evalue, "pred_class" = as.factor(metaeuk_predRed_res_df$Type_cat))
    roc_obj = roc(data = roc_df, response = "pred_class", predictor = "metaeuk_evalue")
    roc_list[[organism_name]] = roc_obj
    
    legend_labels[[organism_name]] = paste0(get_scientific_name(organism_name), " (", sprintf("%.2f", roc_obj$auc), ")")
  }
  
  rocs_df = get_ggroc_df(roc_list)
  
  rocPlot = ggplot(rocs_df, aes(x = fpr, y = min_tpr, colour = name)) +
    geom_step() +
    scale_x_continuous(name = "False Positive Rate", limits = c(0.0,1)) + 
    scale_y_continuous(name = "True Positive Rate", limits = c(0.0,1)) +
    coord_equal() + 
    theme(legend.position = c(0.15, 0.2), legend.text = element_text(face = "italic")) +
    scale_color_discrete(name = "Organism (AUC)", labels = legend_labels)

  return(rocPlot)
}

##################

get_ggpr_df = function(prs) {
  prVals = plyr::ldply(names(prs), function(prName) {
    data.frame(
      #precision = round(prs[[prName]]$curve[,1], digits = 2),
      #recall = round(prs[[prName]]$curve[,2], digits = 2),
      precision = c(1,round(prs[[prName]]$curve[,1], digits = 2),0),
      recall = c(0,round(prs[[prName]]$curve[,2], digits = 2),1),
      name = prName
    )
  })
  
  return(prVals %>% group_by(name, recall) %>% summarise(min_precision=min(precision), max_precision=max(precision)))
}

##################

draw_mapping_pr = function(organism_names) {
  
  legend_labels = list()
  pr_list = list()
  
  for (j in 1:length(organism_names)) {
    organism_name = organism_names[j]
    
    basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
    metaeuk_predRed_res_file = paste(basic_path, "predRed_map_to_annots.tsv",sep = "")
    metaeuk_predRed_res_df = read.table(file = metaeuk_predRed_res_file, header = TRUE, sep = "\t")
    
    metaeuk_predRed_res_df$Type_cat = 0
    metaeuk_predRed_res_df$Type_cat[which(metaeuk_predRed_res_df$Type == "mapped_to_prot")] = 1
    
    # usually class0 is the positives but we define positive and negative the other way around 
    # because e-values are better when smaller
    pr<-pr.curve(scores.class0 = metaeuk_predRed_res_df$evalue[which(metaeuk_predRed_res_df$Type_cat == 0)], 
                 scores.class1 = metaeuk_predRed_res_df$evalue[which(metaeuk_predRed_res_df$Type_cat == 1)],
                 curve=TRUE)
    
    pr_list[[organism_name]] = pr
    
    legend_labels[[organism_name]] = paste0(get_scientific_name(organism_name), " (", sprintf("%.2f", pr$auc.integral), ")")
  }
  
  prs_df = get_ggpr_df(pr_list)
  
  prPlot = ggplot(prs_df, aes(x = recall, y = max_precision, colour = name)) +
    geom_step() +
    scale_x_continuous(name = "Recall", limits = c(0.0,1)) + 
    scale_y_continuous(name = "Precision", limits = c(0.0,1)) +
    coord_equal() + 
    theme(legend.position = c(0.0, 0.2), legend.text = element_text(face = "italic")) +
    scale_color_discrete(name = "AUC-PR", labels = legend_labels)
  
  return(prPlot)
}

##################

draw_target_cov = function(organism_name)
{
  basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
  predRed_SW_file = paste(basic_path, "predRed_SW_and_map_to_annots.tsv",sep = "")
  stats_df = read.table(file = predRed_SW_file, header = TRUE, sep = "\t")
  
  stats_df$map_type_cat = 1
  stats_df$map_type_cat[which(stats_df$map_type == "opp_strand")] = 2
  stats_df$map_type_cat[which(stats_df$map_type == "non_coding")] = 3
  stats_df$map_type_cat[which(stats_df$map_type == "unannot_contig")] = 4
  stats_df$map_type_cat[which(stats_df$map_type == "unmapped")] = 5
  
  my_colors = c('#d7191c' ,'#fdae61' ,'#ffffbf' ,'#abd9e9' ,'#2c7bb6') # 1, 2, 3, 4, 5
  my_labels = c("prot", "prot. on opp. strand", "noncod. element", "unannot. scaff.", "NA")
  levels_inds = sort(unique(stats_df$map_type_cat))
  
  # categorize according to sequence identity with T
  #stats_df$seqid_cat = 0
  #stats_df$seqid_cat[which(stats_df$seqid_to_T > 0.4)] = 1
  #stats_df$seqid_cat[which(stats_df$seqid_to_T > 0.60)] = 2
  #stats_df$seqid_cat[which(stats_df$seqid_to_T > 0.80)] = 3
  #stats_df$seqid_cat = factor(stats_df$seqid_cat, levels = c(3,2,1,0), labels = c("(80%,100%]", "(60%,80%]", "(40%,60%]", "(0%,40%]"))
  #stats_df$seqid_cat = factor(stats_df$seqid_cat, levels = c(3,2,1,0), labels = c("very high", "high", "medium", "low"))
  
  # categorize the target coverage
  stats_df$tcov_cat = 0
  stats_df$tcov_cat[which(stats_df$tcov > 0.50)] = 1
  stats_df$tcov_cat[which(stats_df$tcov > 0.75)] = 2
  stats_df$tcov_cat[which(stats_df$tcov > 0.9)] = 3
  stats_df$tcov_cat = factor(stats_df$tcov_cat, levels = c(3,2,1,0), labels = c(">90%", "(75%,90%]", "(50%,75%]", "<50%"))
  #stats_df$tcov_cat = factor(stats_df$tcov_cat, levels = c(4,3,2,1,0), labels = c("complete", "near complete", "fair", "low", "poor"))
  #stats_df$tcov_cat = factor(stats_df$tcov_cat, levels = c(4,3,2,1,0), labels = c("C", "N", "F", "L", "P"))
  
  tcov_plot = ggplot(stats_df, aes(x=factor(tcov_cat), y=(..count..)/sum(..count..), fill = as.factor(map_type_cat))) + 
    geom_bar() +
    scale_y_continuous(expand=c(0,0), limits = c(0,1), breaks = seq(0,1,by=0.25), labels = scales::percent) + 
    scale_x_discrete(expand=c(0,0), breaks = c(">90%", "(75%,90%]", "(50%,75%]"), limits = c(">90%", "(75%,90%]", "(50%,75%]")) + 
    #theme(axis.text.y = element_blank(), axis.ticks.y = element_blank()) +
    xlab("") + 
    ylab("") +
    scale_fill_manual(values = my_colors[levels_inds], labels = my_labels[levels_inds], breaks = levels_inds, drop=FALSE) +
    #geom_text(aes(label = ifelse((..count..)/sum(..count..) > 0.015, scales::percent((..count..)/sum(..count..), accuracy = 1L), "")), stat="count", position=position_stack(vjust = 0.5)) +
    #theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust=1)) +
    #labs(fill = "Seq. id. to T.") +
    labs(fill = "Mapping status") +
    annotate("text", x = 2.5, y = 0.5, label = get_scientific_name(organism_name), fontface = 'italic')
  
  return (tcov_plot)
}

##################

draw_evalue_bitscore_corr_with_sw = function(organism_name)
{
  basic_path = paste(metaeuk_results_path, organism_name, "/", evaluation_dir, "/", sep ="")
  predRed_SW_file = paste(basic_path, "predRed_SW.tsv",sep = "")
  stats_df = read.table(file = predRed_SW_file, header = TRUE, sep = "\t")
  
  bitscore_spearman_cor = cor(stats_df$metaeuk_bits,stats_df$sw_bits,method = "spearman")
  evalue_spearman_cor = cor(stats_df$metaeuk_evalue,stats_df$sw_evalue,method = "spearman")
  
  rounded_bitscore_cor = round(bitscore_spearman_cor,digits = 2)
  rounded_evalue_cor = round(evalue_spearman_cor,digits = 2)
  
  stats_df$NUM_EXONS_CATEGORY = rep("5+", dim(stats_df)[1])
  stats_df$NUM_EXONS_CATEGORY[which(stats_df$metaeuk_num_exons == 1)] = "1"
  stats_df$NUM_EXONS_CATEGORY[which(stats_df$metaeuk_num_exons == 2)] = "2"
  stats_df$NUM_EXONS_CATEGORY[which(stats_df$metaeuk_num_exons == 3)] = "3"
  stats_df$NUM_EXONS_CATEGORY[which(stats_df$metaeuk_num_exons == 4)] = "4"

  my_colors = c('#fec44f','#fe9929','#ec7014','#cc4c02','#8c2d04') # 1, 2, 3, 4, 5+
  ### draw bitscore
  bitscore_gplot = ggplot(stats_df, aes(x=metaeuk_bits, y=sw_bits)) + 
    geom_point(aes(colour=as.factor(NUM_EXONS_CATEGORY))) +
    scale_y_continuous(limits = c(0,8000), breaks = c(0,4000,8000), labels = c(0,4000,8000), expand = c(0,0)) +
    scale_x_continuous(limits = c(0,8000), breaks = c(0,4000,8000), labels = c(0,4000,8000), expand = c(0,0)) +
    geom_abline(intercept = 0, slope = 1, color="gray", linetype="dashed", size=1) +
    scale_colour_manual(values = my_colors, name = "# exons", labels = c("1", "2", "3", "4", "5+"), breaks = c("1", "2", "3", "4", "5+")) +
    annotate("text", x = 4000, y = 7500, label = get_scientific_name(organism_name), fontface = 'italic') +
    annotate(label = paste0("rho == ", rounded_bitscore_cor), parse = TRUE, "text", x = 4500, y = 1000) +
    xlab("") + 
    ylab("") +
    theme(
      legend.box.margin=margin(c(5,0,-5,0)),
      legend.direction='horizontal',
      legend.spacing = unit(0, "cm")
    )
  
  ### draw log evalue
  evalue_gplot = ggplot(stats_df, aes(x=log(metaeuk_evalue), y=log(sw_evalue))) + 
    geom_point(aes(colour=as.factor(NUM_EXONS_CATEGORY))) +
    scale_y_continuous(limits = c(-800,0), expand = c(0,0)) +
    scale_x_continuous(limits = c(-800,0), expand = c(0,0)) +
    geom_abline(intercept = 0, slope = 1, color="gray", linetype="dashed", size=1) +
    scale_colour_manual(values = my_colors, name = "# exons", labels = c("1", "2", "3", "4", "5+"), breaks = c("1", "2", "3", "4", "5+")) +
    annotate("text", x = -400, y = -50, label = get_scientific_name(organism_name), fontface = 'italic') +
    annotate(label = paste0("rho == ", rounded_evalue_cor), parse = TRUE, "text", x = -200, y = -700) +
    xlab("") + 
    ylab("") +
    theme(
      legend.box.margin=margin(c(5,0,-5,0)),
      legend.direction='horizontal',
      legend.spacing = unit(0, "cm")
    )
  
  figuresList = list("SW_EVAL" = evalue_gplot, "SW_BS" = bitscore_gplot)
  return (figuresList)
}

############################################################################################################
############################################################################################################
########################################## produce the figures #############################################
############################################################################################################
############################################################################################################

basic_path = metaeuk_results_path
dir.create(file.path(path_for_plots), showWarnings = FALSE)

# Panel A #
figure_sensitivity_at_distance = draw_sensitivity_at_distance(organism_names, tot_num_proteins)

# Panel B #
figures_sensitivity_exons = list()
for (i in 1:length(organism_names)) {
  organism_name = organism_names[i]
  total_num_proteins_organism = tot_num_proteins[i]
  figures_sensitivity_exons[[i]] = draw_covered_exons(organism_name,total_num_proteins_organism)
}

# combine A & B #
A_B_grid = plot_grid(
  figure_sensitivity_at_distance + theme(legend.position="none"),
  figures_sensitivity_exons[[1]] + xlab(label = "# exons in annotated protein") + ylab(label = "MetaEuk predictions"),
  figures_sensitivity_exons[[2]],
  figures_sensitivity_exons[[3]] + draw_plot(exon_covered_legend, x=3.25, y=0.2),
  figures_sensitivity_exons[[4]] + ylab(""), 
  figures_sensitivity_exons[[5]],
  figures_sensitivity_exons[[6]],
  figures_sensitivity_exons[[7]],
  labels = c("A", "B", "", "", "", "", "", ""),
  label_x = c(-0.01, 0, 0, 0, 0, 0, 0, 0),
  nrow = 2, ncol = 4
)
A_B_grid_legend = plot_grid(
  A_B_grid,
  NULL,
  get_legend(figure_sensitivity_at_distance + guides(color=guide_legend(ncol=4, byrow=TRUE, title.position = "left", override.aes = list("linetype" = 0))) + theme(legend.position=c(0.1, 0.2))),
  nrow=3, ncol=1, rel_heights=c(0.80,-0.1,0.15)
)

# Panel C #
figure_auc_pr = draw_mapping_pr(organism_names) + guides(color=guide_legend(ncol=1)) + theme(legend.position = c(0.05, 0.4))

# Panel D #
figure_redundancy_reduction_in_grouping = draw_redundancy_reduction(organism_names)

# Panel E #
figure_evalues_Acaste = (draw_evalue_bitscore_corr_with_sw("Acaste"))$SW_EVAL

# combine C, D & E #
C_D_E_grid = plot_grid(
  figure_auc_pr,
  figure_redundancy_reduction_in_grouping,
  figure_evalues_Acaste + theme(legend.position="top") + xlab(label = "Log MetaEuk E-value") + ylab(label = "Log S-W E-value"),
  labels = c("C", "D", "E"),
  nrow=1, ncol=3
)

# combine all #
A_B_C_D_E_grid <- plot_grid(
  A_B_grid_legend,
  C_D_E_grid,
  nrow=2, ncol=1, rel_heights = c(0.66,0.33)
)

### save benchmark figure ###
eval_png = paste(path_for_plots, "evaluation_panels.png", sep = "")
eval_pdf = paste(path_for_plots, "evaluation_panels.pdf", sep = "")

ggsave2(eval_png, A_B_C_D_E_grid, width = 170 * 2, height = 120 * 2, units = c("mm"))
ggsave2(eval_pdf, A_B_C_D_E_grid, width = 170 * 2, height = 120 * 2, units = c("mm"))

##################

figures_tcov = list()
for (i in 1:length(organism_names)) {
  figures_tcov[[i]] = draw_target_cov(organism_names[i])
}

tcov_grid_A_B = plot_grid(
  figures_tcov[[1]] + theme(legend.position="none") + xlab("Target coverage") + ylab("MetaEuk preds."),
  figures_tcov[[2]] + theme(legend.position="none") + theme(axis.text=element_blank()) + xlab("\n") + ylab("\n"),
  nrow = 1, ncol = 2
)

tcov_grid_rest = plot_grid(
  figures_tcov[[3]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  figures_tcov[[4]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  figures_tcov[[5]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  figures_tcov[[6]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  figures_tcov[[7]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  get_legend(figures_tcov[[3]] + theme(legend.direction="vertical", legend.position = c(0.1, 0.6))),
  nrow = 2, ncol = 3
)

tcov_grid = plot_grid(
  tcov_grid_A_B,
  tcov_grid_rest,
  nrow = 2, ncol = 1, rel_heights = c(0.35,0.65)
)

tcov_png = paste(path_for_plots, "tcov.png", sep = "")
tcov_pdf = paste(path_for_plots, "tcov.pdf", sep = "")

ggsave(tcov_png, plot = tcov_grid, width = 170, height = 170, units = c("mm"))
ggsave(tcov_pdf, plot = tcov_grid, width = 170, height = 170, units = c("mm"))

##################

figures_num_exons = list()
figures_exon_len = list()
for (i in 1:length(organism_names)) {
  both_figs = draw_exon_num_length_dist(organism_names[i])
  figures_num_exons[[i]] = both_figs[["NUM_EXONS"]]
  figures_exon_len[[i]] = both_figs[["EXON_LEN"]]
}

num_exons_grid = plot_grid(
  figures_num_exons[[1]] + theme(legend.position="none") + xlab("# putative exons") + ylab("% MetaEuk preds."),
  figures_num_exons[[2]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_num_exons[[3]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_num_exons[[4]] + theme(legend.position="none") +  ylab(""),
  figures_num_exons[[5]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_num_exons[[6]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_num_exons[[7]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  nrow = 2, ncol = 4
)

exon_len_grid = plot_grid(
  figures_exon_len[[1]] + theme(legend.position="none") + xlab("Putative exon len. (AA)") + theme(axis.text.y=element_blank()),
  figures_exon_len[[2]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_exon_len[[3]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_exon_len[[4]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_exon_len[[5]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_exon_len[[6]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  figures_exon_len[[7]] + theme(legend.position="none") + theme(axis.text.y=element_blank()),
  get_legend(figures_exon_len[[3]] + theme(legend.direction="vertical", legend.position = c(0.1, 0.6))),
  nrow = 2, ncol = 4
)

# combine all #
num_len_exon_grid <- plot_grid(
  NULL,
  num_exons_grid,
  exon_len_grid,
  labels = c("A","", "B"),
  nrow=3, ncol=1, rel_heights = c(0.04,0.48,0.48)
)

exon_num_len_png = paste(path_for_plots, "exon_num_len.png", sep = "")
exon_num_len_pdf = paste(path_for_plots, "exon_num_len.pdf", sep = "")

ggsave(exon_num_len_png, plot = num_len_exon_grid, width = 170, height = 200, units = c("mm"))
ggsave(exon_num_len_pdf, plot = num_len_exon_grid, width = 170, height = 200, units = c("mm"))

##################

bitscore_figures = list()
evalue_figures = list()
for (i in 1:length(organism_names))
{
  organism_name = organism_names[i]
  organism_figures_i = draw_evalue_bitscore_corr_with_sw(organism_name)
  bitscore_figures[[i]] = organism_figures_i$SW_BS
  evalue_figures[[i]] = organism_figures_i$SW_EVAL
}

evalue_grid = plot_grid(
  evalue_figures[[1]] + theme(legend.position="none") + xlab("Log MetaEuk E-val") + ylab("Log S-W E-val") + theme(axis.text=element_blank()),
  evalue_figures[[2]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  evalue_figures[[3]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  evalue_figures[[4]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  evalue_figures[[5]] + theme(legend.position="none") + ylab("") + theme(axis.text=element_blank()),
  evalue_figures[[6]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  evalue_figures[[7]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  get_legend(evalue_figures[[2]] + theme(legend.direction="vertical", legend.position = c(0.5, 0.6))),
  nrow = 2, ncol = 4
)

bitscore_grid = plot_grid(
  bitscore_figures[[1]] + theme(legend.position="none") + xlab("MetaEuk bitscore") + ylab("S-W bitscore") + theme(axis.text=element_blank()),
  bitscore_figures[[2]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  bitscore_figures[[3]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  bitscore_figures[[4]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  bitscore_figures[[5]] + theme(legend.position="none") + ylab("") + theme(axis.text=element_blank()),
  bitscore_figures[[6]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  bitscore_figures[[7]] + theme(legend.position="none") + theme(axis.text=element_blank()),
  get_legend(bitscore_figures[[2]] + theme(legend.direction="vertical", legend.position = c(0.5, 0.6))),
  nrow = 2, ncol = 4
)

# combine all #
evals_bitscores_grid <- plot_grid(
  NULL,
  evalue_grid,
  bitscore_grid,
  labels = c("A","", "B"),
  nrow=3, ncol=1, rel_heights = c(0.04,0.48,0.48)
)

evals_bitscores_png = paste(path_for_plots, "evalues_bitscores.png", sep = "")
evals_bitscores_pdf = paste(path_for_plots, "evalues_bitscores.pdf", sep = "")

ggsave(evals_bitscores_png, plot = evals_bitscores_grid, width = 170, height = 200, units = c("mm"))
ggsave(evals_bitscores_pdf, plot = evals_bitscores_grid, width = 170, height = 200, units = c("mm"))
