use strict;
use warnings;

# see info about gff3 format here: https://github.com/The-Sequence-Ontology/Specifications/blob/master/gff3.md
# and here: https://www.ncbi.nlm.nih.gov/genbank/genomes_gff/

my ($in_gff_file, $in_proteins_fasta_file, $out_annot_protein_coords_file, $out_annot_non_coding_gene_features_file) = @ARGV;
# the protein fasta file is only used to validate the GFF

my %proteins_from_fasta_hash;
get_protein_ids_from_fasta($in_proteins_fasta_file, \%proteins_from_fasta_hash);

my %coding_genes_hash;
get_coding_genes($in_gff_file, \%coding_genes_hash);
my %children_of_coding_genes_hash;
get_children_of_coding_genes($in_gff_file, \%coding_genes_hash, \%children_of_coding_genes_hash);
my %parsed_gff_hash;
parse_gff($in_gff_file, \%parsed_gff_hash, \%coding_genes_hash, \%children_of_coding_genes_hash);

print_feutures_as_metaeuk_headers(\%parsed_gff_hash, \%proteins_from_fasta_hash, $out_annot_protein_coords_file, $out_annot_non_coding_gene_features_file);

sub get_protein_ids_from_fasta {
	my ($in_proteins_fasta_file, $protein_hash_ref) = @_;
	open (my $in, "<", $in_proteins_fasta_file) or die "could not open $in_proteins_fasta_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		if ($line =~ m/^>(\S+)/) {
			$protein_hash_ref->{$1} = 0;
		}
		$line = <$in>;
	}
	
	my $num_proteins = keys %{$protein_hash_ref};
	print "found $num_proteins in fasta file\n";
}

sub print_feutures_as_metaeuk_headers {
	my ($gff_hash_ref, $protein_hash_ref, $out_annot_protein_coords_file, $out_annot_non_coding_gene_features_file) = @_;
	
	############### print protein coding genes ###############
	my @protein_lines_to_print;
	foreach my $gene_ID (keys %{$gff_hash_ref->{'coding_gene'}}) {
		my $gene_contig = $gff_hash_ref->{'coding_gene'}{$gene_ID}{"contig"};
		my $gene_start = $gff_hash_ref->{'coding_gene'}{$gene_ID}{"start"};
		my $gene_end = $gff_hash_ref->{'coding_gene'}{$gene_ID}{"end"};
		my $gene_strand = $gff_hash_ref->{'coding_gene'}{$gene_ID}{"strand"};		
		my @gene_CDSs_array = @{$gff_hash_ref->{'coding_gene'}{$gene_ID}{"CDSs"}};
		my %protein_to_CDSs;
		foreach my $CDS_hash_ref (@gene_CDSs_array) {
			my $protein = $CDS_hash_ref->{"protein"};
			
			# validate protein is in protein fasta file
			if (! exists $protein_hash_ref->{$protein}) {
				die "Error: could not file $protein in protein fasta file\n";
			}
			
			if (! exists $protein_to_CDSs{$protein}) {
				$protein_to_CDSs{$protein} = [$CDS_hash_ref];
			} else {
				push (@{$protein_to_CDSs{$protein}}, $CDS_hash_ref);
			}
		}
		foreach my $protein_for_gene (keys %protein_to_CDSs) {
			
			# indicate that this protein was seen in the GFF
			$protein_hash_ref->{$protein_for_gene} += 1;
				
			#T_header|C_header|S|bitscore|E-Value|number_exons|low_coord|high_coord|exon1_coords|exon2_coords|...
			#The exon_coords are of the structure: low[taken_low]:high[taken_high]:nucleotide_length[taken_nucleotide_length]
			# NOTE - here taken_nucleotide_length may not be divisible by 3 (most of the time it will be)
			my @protein_CDSs_array = @{$protein_to_CDSs{$protein_for_gene}};
			my $num_CDSs = @protein_CDSs_array;
			my @CDS_str_arr;
			foreach my $CDS_hash_ref (@protein_CDSs_array) {
				my $CDS_start = $CDS_hash_ref->{"start"};
				my $CDS_end = $CDS_hash_ref->{"end"};
				my $CDS_length = $CDS_end - $CDS_start + 1;
				push (@CDS_str_arr, "$CDS_start\[$CDS_start\]:$CDS_end\[$CDS_end\]:$CDS_length\[$CDS_length\]");
			}
			my $CDSs_str = join("\|", @CDS_str_arr);
			push (@protein_lines_to_print, ">$protein_for_gene\|$gene_contig\|$gene_strand\|0\|0\|$num_CDSs\|$gene_start\|$gene_end\|$CDSs_str");
		}
	}
	
	# check that all fasta proteins were seen exactly once
	foreach my $protein (keys %{$protein_hash_ref}) {
		if ($protein_hash_ref->{$protein} != 1) {
			die "Error: saw $protein more/less than once if GFF\n";
		}
	}
	
	# everything is okay - print output
	open (my $out, ">", $out_annot_protein_coords_file) or die "could not open $out_annot_protein_coords_file for writing\n";
	foreach my $line (@protein_lines_to_print) {
		print $out "$line\n";
	}
	close ($out);
	
	
	############### print non coding features ###############
	open (my $out_noncoding, ">", $out_annot_non_coding_gene_features_file) or die "could not open $out_annot_non_coding_gene_features_file for writing\n";
	
	foreach my $feature_ID (keys %{$gff_hash_ref->{'noncoding'}}) {
		my $feature_contig = $gff_hash_ref->{'noncoding'}{$feature_ID}{"contig"};
		my $feature_start = $gff_hash_ref->{'noncoding'}{$feature_ID}{"start"};
		my $feature_end = $gff_hash_ref->{'noncoding'}{$feature_ID}{"end"};
		my $feature_strand = $gff_hash_ref->{'noncoding'}{$feature_ID}{"strand"};
		my $feature_type = $gff_hash_ref->{'noncoding'}{$feature_ID}{"type"};
		
		print $out_noncoding ">$feature_type\|$feature_contig\|$feature_strand\|0\|0\|0\|$feature_start\|$feature_end\n";
	}
	close ($out_noncoding);
}

sub parse_gff {
	my ($in_gff_file, $gff_hash_ref, $coding_genes_hash_ref, $children_to_coding_genes_hash_ref) = @_;
	
	open (my $in, "<", $in_gff_file) or die "could not open $in_gff_file for reading\n";
	
	my $gff_line = <$in>;
	while (defined $gff_line) {
		chomp $gff_line;
		# if not a comment line - it has 9 fields (checked by get_children_of_genes)
		if ($gff_line !~ m/^\#/) {
			my @fields = split (/\t/,$gff_line);
			
			# extract relevant info
			my $feature_contig = $fields[0];
			my $feature_type = $fields[2];
			my $feature_start = $fields[3]; # 1-based integer coordinates
			my $feature_end = $fields[4]; # 1-based integer coordinates, always greater than or equal to start
			my $feature_strand = $fields[6];
			my $feature_phase = $fields[7]; # relevant only to CDS. remove this many nucleotides to get to the first codon. 
			# for forward strand features, phase is counted from the start field. For reverse strand features, phase is counted from the end field.
			# for eukaryotes the GenBank: multiple CDS rows with the same Parent attribute represent parts of the same CDS feature
			# so there is always a 1:1 relationship of mRNA to CDS
			# CDS feature_IDs are not unique
			
			my $feature_ID = get_val_from_attributes_field($fields[8], 'ID');
			my $feature_parent = get_val_from_attributes_field($fields[8], 'Parent');
			my $feature_biotype = get_val_from_attributes_field($fields[8], 'gene_biotype');

			# if this is a protein coding gene
			if (($feature_type eq 'gene') and (exists $coding_genes_hash_ref->{$feature_ID})) {
				my $gene_ID = $feature_ID;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"contig"} = $feature_contig;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"start"} = $feature_start;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"end"} = $feature_end;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"strand"} = $feature_strand;
				$gff_hash_ref->{'coding_gene'}{$feature_ID}{"biotype"} = $feature_biotype;
			}
			# if this is an mRNA and its parent is a protein coding gene
			elsif (($feature_type eq 'mRNA') and (exists $coding_genes_hash_ref->{$feature_parent})) {
				my $gene_ID = $feature_parent;
				my $mRNA_ID = $feature_ID;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"mRNAs"}{$mRNA_ID}{"start"} = $feature_start;
				$gff_hash_ref->{'coding_gene'}{$gene_ID}{"mRNAs"}{$mRNA_ID}{"end"} = $feature_end;
			}
			# or if this a CDS (usually a child of an mRNA, which is a child of a gene but sometime direct child of a gene)
			elsif ($feature_type eq 'CDS') {
				my $gene_ID = "";
				my $mRNA_ID = "no_mRNA_level";
				if (exists $children_to_coding_genes_hash_ref->{$feature_parent}) {
					$mRNA_ID = $feature_parent;
					$gene_ID = $children_to_coding_genes_hash_ref->{$feature_parent};
				} elsif (exists $coding_genes_hash_ref->{$feature_parent}) {
					$gene_ID = $feature_parent;
				} else {
					my $is_pseudo = get_val_from_attributes_field($fields[8], 'pseudo');
					if (($is_pseudo eq "true") and ($feature_parent ne "NA")) {
						print "Found pseudo-CDS, case handeled by parent level\n";
					} else {
						warn "Warning: found CDS without mRNA nor gene parent. Full GFF line:\n$gff_line\nSkipping...\n";
					}
					$gff_line = <$in>;
					next;
				}
				
				my $protein_ID = get_val_from_attributes_field($fields[8], 'protein_id');
				# correct for phase - note that even after correction may not divide by three (casue an incomplete codon might remain at the end)
				if ($feature_strand eq "+") {
					$feature_start += $feature_phase;
				} else {
					$feature_end -= $feature_phase;
				}
				# since CDS ids are not unique, keep them in an array
				if (! exists $gff_hash_ref->{'coding_gene'}{$gene_ID}{"CDSs"}) {
					$gff_hash_ref->{'coding_gene'}{$gene_ID}{"CDSs"} = [{"start" => $feature_start, "end" => $feature_end, "protein" => $protein_ID}];
				} else {
					push (@{$gff_hash_ref->{'coding_gene'}{$gene_ID}{"CDSs"}}, {"start" => $feature_start, "end" => $feature_end, "protein" => $protein_ID});
				}
			}
			# for non-coding, retain only the parentless level
			elsif (($feature_parent eq 'NA') and ($feature_type ne 'region') and ($feature_type ne 'sequence_feature')) {
				$gff_hash_ref->{'noncoding'}{$feature_ID}{"contig"} = $feature_contig;
				$gff_hash_ref->{'noncoding'}{$feature_ID}{"start"} = $feature_start;
				$gff_hash_ref->{'noncoding'}{$feature_ID}{"end"} = $feature_end;
				$gff_hash_ref->{'noncoding'}{$feature_ID}{"strand"} = $feature_strand;
				$gff_hash_ref->{'noncoding'}{$feature_ID}{"type"} = $feature_type;
				if ($feature_biotype ne "NA") {
					# more specific description
					$gff_hash_ref->{'noncoding'}{$feature_ID}{"type"} = $feature_biotype;
				}
			}
		}
		# read the next line
		$gff_line = <$in>;
	}
	close ($in);
}

sub get_children_of_coding_genes {
	my ($in_gff_file, $coding_genes_hash_ref, $children_to_coding_genes_hash_ref) = @_;
	open (my $in, "<", $in_gff_file) or die "could not open $in_gff_file for reading\n";

	my $gff_line = <$in>;
	while (defined $gff_line) {
		chomp $gff_line;
		# if not a comment line - it has 9 fields (checked by get_coding_genes)
		if ($gff_line !~ m/^\#/) {
			my @fields = split (/\t/,$gff_line);
			my $feature_type = $fields[2];
			my $feature_ID = get_val_from_attributes_field($fields[8], 'ID');
			my $feature_parent = get_val_from_attributes_field($fields[8], 'Parent');
			
			if (exists $coding_genes_hash_ref->{$feature_parent}) {
				$children_to_coding_genes_hash_ref->{$feature_ID} = $feature_parent;
			}
		}
		# read the next line
		$gff_line = <$in>;
	}
	close ($in);
}


sub get_coding_genes {
	my ($in_gff_file, $coding_genes_hash_ref) = @_;
	open (my $in, "<", $in_gff_file) or die "could not open $in_gff_file for reading\n";
	
	my $num_protein_coding_genes = 0;
	
	my $gff_line = <$in>;
	while (defined $gff_line) {
		chomp $gff_line;
		# if not a comment line - has to have 9 fields
		if ($gff_line !~ m/^\#/) {
			my @fields = split (/\t/,$gff_line);
			if (@fields != 9) {
				die "Error in gff.v3 format: number of fields in info line is not 9: '$gff_line'\n";
			}
			my $feature_type = $fields[2];
			my $feature_ID = get_val_from_attributes_field($fields[8], 'ID');
			my $feature_biotype = get_val_from_attributes_field($fields[8], 'gene_biotype');
			
			if (($feature_type eq 'gene') and ($feature_biotype eq 'protein_coding')) {
				$coding_genes_hash_ref->{$feature_ID} = 1;
				$num_protein_coding_genes++;
			}	
		}
		# read the next line
		$gff_line = <$in>;
	}
	close ($in);
	
	print "found $num_protein_coding_genes protein_coding genes\n";
}

sub get_val_from_attributes_field {
	my ($attribute_field, $requested_tag) = @_;
	my @attribute_pairs = split (/\;/, $attribute_field);
	foreach my $attribute_pair (@attribute_pairs) {
		my ($tag, $val) = split (/\=/, $attribute_pair);
		if ($tag eq $requested_tag) {
			return ($val);
		}	
	}
	return "NA";
}