library(ggplot2)
library(scales)
library(plyr)
library(ggridges)

# define default font sizes, etc.
theme_set(theme_classic(
  base_size = 11
))

args <- commandArgs(TRUE)
contigs_num_preds_file <- as.character(args[1])
numExonsFile <- as.character(args[2])
path_for_plot <- as.character(args[3])

## for debug:
contigs_num_preds_file="tara/run_011/predRed_Tara5kbEukRep_predictions_per_contig_stats.txt"
numExonsFile="tara/run_011/predRed_Tara5kbEukRep_num_and_length_exon_dist.txt"
path_for_plot="tara/run_011/plots/predRed_Tara5kbEukRep_contigs_preds_and_exons_stats"

dir.create(file.path("tara/run_011/plots/"), showWarnings = FALSE)

path_for_plot_png = paste(path_for_plot,".png",sep = "")
path_for_plot_pdf = paste(path_for_plot,".pdf",sep = "")

#####
contigs_df = read.csv(file = contigs_num_preds_file, header = TRUE, sep = "\t")
sorted_lengths = sort(contigs_df$LENGTH)
num_contigs = length(sorted_lengths)
max_of_25_percent = sorted_lengths[num_contigs * 0.25]
max_of_50_percent = sorted_lengths[num_contigs * 0.5]
max_of_75_percent = sorted_lengths[num_contigs * 0.75]

max_num_preds = max(contigs_df$NUM_PREDS)
max_contig_length = max(contigs_df$LENGTH)
set.seed(3)
contigs_sampled_df = contigs_df[sample(nrow(contigs_df), size = 10000, replace = FALSE),]

fig_preds_per_contig = ggplot(contigs_sampled_df, aes(x=log(LENGTH), y=log(1+NUM_PREDS))) +
  geom_point(color = "gray", size = 0.7, alpha = 0.9) +
  scale_x_continuous(limits = log(c(5000,200000)), breaks = log(c(5000,7000,10000,20000,50000,100000)), labels = comma(c(5000,7000,10000,20000,50000,100000)), expand=c(0,0)) +
  scale_y_continuous(limits = log(c(1,250)), breaks = log(1+c(0,1,2,3,5,10,20,100,200)), labels = comma(c(0,1,2,3,5,10,20,100,200))) +
  xlab("Contig length (bp)") +
  ylab("# MetaEuk predictions") +
  geom_vline(aes(xintercept = log(max_of_25_percent)), color = "#444444", linetype="dashed") +
  geom_vline(aes(xintercept = log(max_of_50_percent)), color = "#444444", linetype="dashed") +
  geom_vline(aes(xintercept = log(max_of_75_percent)), color = "#444444", linetype="dashed") +
  annotate("label", x = log(max_of_25_percent), y = log(210), label = "Q1", size = 3) +
  annotate("label", x = log(max_of_50_percent), y = log(210), label = "Q2", size = 3) +
  annotate("label", x = log(max_of_75_percent), y = log(210), label = "Q3", size = 3) +
  stat_density_2d(aes(fill = stat(nlevel)), geom = "polygon") +
  scale_fill_continuous("", limits = c(0,1), breaks = c(0,1), labels = c("Low", "High"), expand = c(0,0)) +
  theme(
    legend.position = c(0.4,0.9),
    legend.direction = "horizontal"
  ) +
  annotate("text", x = log(12300), y = log(210), label = "Density", size = 3)
  
#ggsave(paste0(path_for_plot,"_preds_per_contig.pdf"), fig_preds_per_contig, width = 170, height = 100, units = c("mm"))

#######

numExons = read.delim(file = numExonsFile, header = TRUE, sep = "\t")
lengthExonsSingleExonPreds = numExons$length_each_exon[which(numExons$num_exons == 1)]
lengthExonsMultiExonPreds = numExons$length_each_exon[which(numExons$num_exons > 1)]
lengthExonsDf = data.frame(LENGTH = c(lengthExonsSingleExonPreds,lengthExonsMultiExonPreds), GROUP = c(rep("Single",length(lengthExonsSingleExonPreds)),rep("Multi",length(lengthExonsMultiExonPreds))))
numPredsSingleExon = length(which(numExons$num_exons == 1))
numPredsTwoExons = length(which(numExons$num_exons == 2)) / 2
totalNumPreds = length(unique(as.numeric(numExons$pred_id)))
numPredsThreeOrMoreExons = totalNumPreds - (numPredsTwoExons + numPredsSingleExon)

lengthExonsSampledDf = lengthExonsDf[sample(nrow(lengthExonsDf), size = 10000, replace = FALSE),]
figExonsLength = ggplot(lengthExonsSampledDf, aes(x=log(LENGTH), y=GROUP, fill=GROUP, color=GROUP)) + 
  geom_density_ridges(quantile_lines = TRUE, alpha = 0.5, scale = 100000) +
  scale_x_continuous(limits = log(c(9,3000)), breaks = log(c(10,20,30,50,100,300,1000)), labels = comma(c(10,20,30,50,100,300,1000))) +
  xlab("Putative exon length (AAs)") +
  ylab("") +
  scale_y_discrete(breaks = NULL) +
  scale_fill_discrete (
    "",
    breaks=c("Single", "Multi"),
    labels=c("1 putative exon", "2+ putative exons")
  ) +
  scale_color_discrete (
    breaks=NULL
  ) +
  theme(
    legend.position = c(0.8,0.87),
    legend.text = element_text(size = 9),
    legend.key.height = unit(0.003, "npc"),
    legend.key.width = unit(0.01, "npc")
  ) +
  annotate("text", -Inf, Inf, label = 'bold("C")', size = 5, parse = TRUE, hjust = 0, vjust = 1)
  
#ggsave(paste0(path_for_plot,"_exons_length.pdf"), figExonsLength, width = 70, height = 50, units = c("mm"))

#### histogram of predictions by num exons ####

predsByNumExonsDf = data.frame("NUM_EXONS" = c(1,2,3), "PRED_COUNT" = c(numPredsSingleExon, numPredsTwoExons, numPredsThreeOrMoreExons))
totNumPreds = sum(predsByNumExonsDf$PRED_COUNT)
predsByNumExonsDf$PERCENT_PREDS = round(100 * predsByNumExonsDf$PRED_COUNT / totNumPreds, digits = 2)
predsByNumExonsDf$COUNT_LABEL = paste0(round((predsByNumExonsDf$PRED_COUNT / 1000000), digits = 1),"M")

figPredsByNumExons = ggplot(predsByNumExonsDf, aes(y = PERCENT_PREDS, x = as.factor(NUM_EXONS))) + 
  geom_bar(stat="identity", position = "dodge", width=0.8) + 
  geom_text(aes(label = COUNT_LABEL, x = as.factor(NUM_EXONS)), position = position_dodge(width = 0.8), vjust = -0.6, size = 3.5) +
  scale_y_continuous(breaks = seq(0, 100, by = 20), limits = c(0,100), labels = scales::percent(seq(0, 1, by = 0.2)), expand = c(0,0)) +
  scale_x_discrete(breaks = c(1,2,3), labels = c("1","2","3+")) +
  xlab("# putative exons") + 
  ylab("MetaEuk preds.") +
  annotate("text", x = 3, y = 90, label = 'bold("B")', size = 5, parse = TRUE)

#ggsave(paste0(path_for_plot,"_preds_by_num_exons.pdf"), figPredsByNumExons, width = 45, height = 45, units = c("mm"))


#### adding the panels as insets ####

figExonsLengthGrob = ggplotGrob(x = figExonsLength)
figPredsByNumExonsGrob = ggplotGrob(x = figPredsByNumExons)

fig_preds_per_contig_with_insets = fig_preds_per_contig +
  annotation_custom (
    grob = figExonsLengthGrob,
    xmin = log(35000),
    xmax = Inf,
    ymin = -Inf,
    ymax = log(8.5)
  ) +
  annotation_custom (
    grob = figPredsByNumExonsGrob,
    xmin = log(70000),
    xmax = Inf,
    ymin = log(8.5),
    ymax = log(140)
  )


fig_preds_per_contig_with_insets_arranged = grid.arrange(grobs = list(arrangeGrob(fig_preds_per_contig_with_insets,
                                      top = textGrob("A", x = unit(0.02, "npc"), y = unit(0.6, "npc"), just=c("left","top"), gp=gpar(fontface="bold", col="black", fontsize=14)),
                                      ncol=1,nrow=1)))

ggsave(paste0(path_for_plot,".pdf"), fig_preds_per_contig_with_insets_arranged, width = 170, height = 100, units = c("mm"))
ggsave(paste0(path_for_plot,".png"), fig_preds_per_contig_with_insets_arranged, width = 170, height = 100, units = c("mm"))

